Class: Effective::Order

Inherits:
ActiveRecord::Base
  • Object
show all
Defined in:
app/models/effective/order.rb

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(atts = nil, &block) ⇒ Order

Effective::Order.new(items: Product.first) Effective::Order.new(items: [Product.first, Product.second], user: User.first) Effective::Order.new(items: Product.first, user: User.first, billing_address: Effective::Address.new, shipping_address: Effective::Address.new)



206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
# File 'app/models/effective/order.rb', line 206

def initialize(atts = nil, &block)
  super(state: EffectiveOrders::PENDING) # Initialize with state: PENDING

  return self unless atts.present?

  if atts.kind_of?(Hash)
    items = Array(atts[:item]) + Array(atts[:items])

    self.user = atts[:user] || items.first.try(:user)

    if (address = atts[:billing_address]).present?
      self.billing_address = address
      self.billing_address.full_name ||= user.to_s.presence
    end

    if (address = atts[:shipping_address]).present?
      self.shipping_address = address
      self.shipping_address.full_name ||= user.to_s.presence
    end

    atts.except(:item, :items, :user, :billing_address, :shipping_address).each do |key, value|
      self.send("#{key}=", value)
    end

    add(items) if items.present?
  else # Attributes are not a Hash
    self.user = atts.user if atts.respond_to?(:user)
    add(atts)
  end

  self
end

Instance Attribute Details

#confirmed_checkout ⇒ Object

Set on the Checkout Step 1



24
25
26
# File 'app/models/effective/order.rb', line 24

def confirmed_checkout
  @confirmed_checkout
end

#send_mark_as_paid_email_to_buyer ⇒ Object

Set by Admin::Orders#mark_as_paid



28
29
30
# File 'app/models/effective/order.rb', line 28

def send_mark_as_paid_email_to_buyer
  @send_mark_as_paid_email_to_buyer
end

#send_payment_request_to_buyer ⇒ Object

Settings in the /admin action forms



27
28
29
# File 'app/models/effective/order.rb', line 27

def send_payment_request_to_buyer
  @send_payment_request_to_buyer
end

#skip_buyer_validations ⇒ Object

Set by Admin::Orders#create



29
30
31
# File 'app/models/effective/order.rb', line 29

def skip_buyer_validations
  @skip_buyer_validations
end

#terms_and_conditions ⇒ Object

Yes, I agree to the terms and conditions



23
24
25
# File 'app/models/effective/order.rb', line 23

def terms_and_conditions
  @terms_and_conditions
end

Instance Method Details

#abandoned? ⇒ Boolean

Returns:

  • (Boolean)


457
458
459
# File 'app/models/effective/order.rb', line 457

def abandoned?
  state == EffectiveOrders::ABANDONED
end

#add(*items, quantity: 1) ⇒ Object

Items can be an Effective::Cart, an Effective::order, a single acts_as_purchasable, or multiple acts_as_purchasables add(Product.first) => returns an Effective::OrderItem add(Product.first, current_cart) => returns an array of Effective::OrderItems



265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
# File 'app/models/effective/order.rb', line 265

def add(*items, quantity: 1)
  raise 'unable to alter a purchased order' if purchased?
  raise 'unable to alter a declined order' if declined?

  cart_items = items.flatten.flat_map do |item|
    if item.kind_of?(Effective::Cart)
      item.cart_items.to_a
    elsif item.kind_of?(ActsAsPurchasable)
      Effective::CartItem.new(quantity: quantity, purchasable: item)
    elsif item.kind_of?(Effective::Order)
      # Duplicate an existing order
      self.note_to_buyer ||= item.note_to_buyer
      self.note_internal ||= item.note_internal
      self.cc ||= item.cc

      item.order_items.select { |oi| oi.purchasable.kind_of?(Effective::Product) }.map do |oi|
        purchasable = oi.purchasable

        product = Effective::Product.new(name: purchasable.purchasable_name, price: purchasable.price, tax_exempt: purchasable.tax_exempt)

        # Copy over any extended attributes that may have been created
        atts = purchasable.dup.attributes.except('name', 'price', 'tax_exempt', 'purchased_order_id').compact

        atts.each do |k, v|
          next unless product.respond_to?("#{k}=") && product.respond_to?(k)
          product.send("#{k}=", v) if product.send(k).blank?
        end

        Effective::CartItem.new(quantity: oi.quantity, purchasable: product)
      end
    else
      raise 'add() expects one or more acts_as_purchasable objects, or an Effective::Cart'
    end
  end.compact

  # Make sure to reset stored aggregates
  assign_attributes(subtotal: nil, tax_rate: nil, tax: nil, amount_owing: nil, surcharge_percent: nil, surcharge: nil, total: nil)

  retval = cart_items.map do |item|
    order_items.build(
      name: item.name,
      quantity: item.quantity,
      price: item.price,
      tax_exempt: (item.tax_exempt || false),
    ).tap { |order_item| order_item.purchasable = item.purchasable }
  end

  retval.size == 1 ? retval.first : retval
end

#amount_owing ⇒ Object



477
478
479
# File 'app/models/effective/order.rb', line 477

def amount_owing
  self[:amount_owing] || get_amount_owing()
end

#assign_confirmed_if_valid! ⇒ Object

This lets us skip to the confirmed workflow for an admin...



579
580
581
582
583
584
585
586
587
588
# File 'app/models/effective/order.rb', line 579

def assign_confirmed_if_valid!
  return unless pending?

  self.state = EffectiveOrders::CONFIRMED
  return true if valid?

  self.errors.clear
  self.state = EffectiveOrders::PENDING
  false
end

#billing_first_name ⇒ Object



401
402
403
# File 'app/models/effective/order.rb', line 401

def billing_first_name
  billing_name.to_s.split(' ').first
end

#billing_last_name ⇒ Object



405
406
407
# File 'app/models/effective/order.rb', line 405

def billing_last_name
  Array(billing_name.to_s.split(' ')[1..-1]).join(' ')
end

#confirm! ⇒ Object

Used by admin checkout only



573
574
575
576
# File 'app/models/effective/order.rb', line 573

def confirm!
  return false if purchased?
  update!(state: EffectiveOrders::CONFIRMED)
end

#confirmed? ⇒ Boolean

Returns:

  • (Boolean)


413
414
415
# File 'app/models/effective/order.rb', line 413

def confirmed?
  state == EffectiveOrders::CONFIRMED
end

#custom_order? ⇒ Boolean

A custom order is one that was created by an admin We allow custom orders to have their order items updated

Returns:

  • (Boolean)


431
432
433
# File 'app/models/effective/order.rb', line 431

def custom_order?
  order_items.all? { |oi| oi.purchasable_type == 'Effective::Product' }
end

#decline!(payment: 'none', provider: 'none', card: 'none', validate: true) ⇒ Object



679
680
681
682
683
684
685
686
687
688
689
690
691
692
693
694
695
696
697
698
699
700
701
702
703
704
705
706
707
708
709
710
711
# File 'app/models/effective/order.rb', line 679

def decline!(payment: 'none', provider: 'none', card: 'none', validate: true)
  return false if declined?

  raise('order already purchased') if purchased?

  error = nil

  assign_attributes(
    state: EffectiveOrders::DECLINED,
    purchased_at: nil,
    payment: payment_to_h(payment),
    payment_provider: provider,
    payment_card: (card.presence || 'none'),
    skip_buyer_validations: true
  )

  Effective::Order.transaction do
    begin
      run_purchasable_callbacks(:before_decline)
      save!(validate: validate)
      run_purchasable_callbacks(:after_decline)
    rescue => e
      self.state = state_was

      error = e.message
      raise ::ActiveRecord::Rollback
    end
  end

  raise "Failed to decline order: #{error || errors.full_messages.to_sentence}" unless error.nil?

  true
end

#declined? ⇒ Boolean

Returns:

  • (Boolean)


453
454
455
# File 'app/models/effective/order.rb', line 453

def declined?
  state == EffectiveOrders::DECLINED
end

#defer!(provider: 'none', email: true) ⇒ Object



668
669
670
671
672
673
674
675
676
677
# File 'app/models/effective/order.rb', line 668

def defer!(provider: 'none', email: true)
  return false if purchased?

  assign_attributes(state: EffectiveOrders::DEFERRED, payment_provider: provider)
  save!

  send_payment_request_to_buyer! if email

  true
end

#deferred? ⇒ Boolean

Returns:

  • (Boolean)


417
418
419
# File 'app/models/effective/order.rb', line 417

def deferred?
  state == EffectiveOrders::DEFERRED
end

#done? ⇒ Boolean

Returns:

  • (Boolean)


425
426
427
# File 'app/models/effective/order.rb', line 425

def done?
  persisted? && (purchased? || declined?)
end

#duplicate ⇒ Object



392
393
394
# File 'app/models/effective/order.rb', line 392

def duplicate
  Effective::Order.new(self)
end

#emails ⇒ Object

These are all the emails we send all notifications to



714
715
716
# File 'app/models/effective/order.rb', line 714

def emails
  ([purchased_by.try(:email)] + [email] + [user.try(:email)] + Array(organization.try(:billing_emails))).map(&:presence).compact.uniq
end

#emails_send_to ⇒ Object

Doesn't control anything. Purely for the flash messaging



719
720
721
# File 'app/models/effective/order.rb', line 719

def emails_send_to
  (emails + [cc.presence]).compact.uniq.to_sentence
end

#free? ⇒ Boolean

Returns:

  • (Boolean)


505
506
507
# File 'app/models/effective/order.rb', line 505

def free?
  total == 0
end

#in_progress? ⇒ Boolean

Returns:

  • (Boolean)


421
422
423
# File 'app/models/effective/order.rb', line 421

def in_progress?
  pending? || confirmed? || deferred?
end

#label ⇒ Object



336
337
338
339
340
341
342
343
344
345
346
347
348
# File 'app/models/effective/order.rb', line 336

def label
  if refund? && purchased?
    'Refund'
  elsif purchased?
    'Receipt'
  elsif refund? && (pending? || confirmed?)
    'Pending Refund'
  elsif (pending? || confirmed?)
    'Pending Order'
  else
    'Order'
  end
end

#mark_as_purchased! ⇒ Object

Call this as a way to skip over non consequential orders And mark some purchasables purchased This is different than the Mark as Paid payment processor



599
600
601
# File 'app/models/effective/order.rb', line 599

def mark_as_purchased!
  purchase!(skip_buyer_validations: true, email: false, skip_quickbooks: true)
end

#num_items ⇒ Object



520
521
522
# File 'app/models/effective/order.rb', line 520

def num_items
  present_order_items.map { |oi| oi.quantity }.sum
end

#payment_method ⇒ Object

Visa - 1234



355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
# File 'app/models/effective/order.rb', line 355

def payment_method
  return nil unless purchased?

  provider = payment_provider if ['cheque', 'etransfer', 'phone', 'credit card'].include?(payment_provider)

  # Normalize payment card
  card = case payment_card.to_s.downcase.gsub(' ', '').strip
    when '' then nil
    when 'v', 'visa' then 'Visa'
    when 'm', 'mc', 'master', 'mastercard' then 'MasterCard'
    when 'a', 'ax', 'american', 'americanexpress' then 'American Express'
    when 'd', 'discover' then 'Discover'
    else payment_card.to_s
  end

  # Try again
  if card == 'none' && payment['card_type'].present?
    card = case payment['card_type'].to_s.downcase.gsub(' ', '').strip
      when '' then nil
      when 'v', 'visa' then 'Visa'
      when 'm', 'mc', 'master', 'mastercard' then 'MasterCard'
      when 'a', 'ax', 'american', 'americanexpress' then 'American Express'
      when 'd', 'discover' then 'Discover'
      else payment_card.to_s
    end
  end

  last4 = if payment[:active_card] && payment[:active_card].include?('**** **** ****')
    payment[:active_card][15,4]
  end

  # stripe, moneris, moneris_checkout
  last4 ||= (payment['f4l4'] || payment['first6last4']).to_s.last(4)

  [provider.presence, card.presence, last4.presence].compact.join(' - ')
end

#pending! ⇒ Object

This is called from admin/orders#create This is intended for use as an admin action only It skips any address or bad user validations It's basically the same as save! on a new order, except it might send the payment request to buyer



558
559
560
561
562
563
564
565
566
567
568
569
570
# File 'app/models/effective/order.rb', line 558

def pending!
  return false if purchased?

  self.state = EffectiveOrders::PENDING
  self.addresses.clear if addresses.any? { |address| address.valid? == false }
  save!

  if send_payment_request_to_buyer?
    after_commit { send_payment_request_to_buyer! }
  end

  true
end

#pending? ⇒ Boolean

Returns:

  • (Boolean)


409
410
411
# File 'app/models/effective/order.rb', line 409

def pending?
  state == EffectiveOrders::PENDING
end

#pending_refund? ⇒ Boolean

Returns:

  • (Boolean)


513
514
515
516
517
518
# File 'app/models/effective/order.rb', line 513

def pending_refund?
  return false if EffectiveOrders.buyer_purchases_refund?
  return false if purchased?

  refund?
end

#purchasables ⇒ Object



461
462
463
# File 'app/models/effective/order.rb', line 461

def purchasables
  present_order_items.map { |order_item| order_item.purchasable }
end

#purchase!(payment: nil, provider: nil, card: nil, email: true, skip_buyer_validations: false, skip_quickbooks: false, current_user: nil) ⇒ Object

Effective::Order.new(items: Product.first, user: User.first).purchase!(email: false)



604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
# File 'app/models/effective/order.rb', line 604

def purchase!(payment: nil, provider: nil, card: nil, email: true, skip_buyer_validations: false, skip_quickbooks: false, current_user: nil)
  return true if purchased?

  # Assign attributes
  assign_attributes(
    state: EffectiveOrders::PURCHASED,
    skip_buyer_validations: skip_buyer_validations,

    purchased_at: (purchased_at.presence || Time.zone.now),
    purchased_by: (purchased_by.presence || current_user),

    payment: payment_to_h(payment.presence || 'none'),
    payment_provider: (provider.presence || 'none'),
    payment_card: (card.presence || 'none')
  )

  if current_user&.email.present?
    assign_attributes(email: current_user.email)
  end

  # Updates surcharge and total based on payment_provider
  assign_order_charges()

  begin
    Effective::Order.transaction do
      run_purchasable_callbacks(:before_purchase)

      save!
      update_purchasables_purchased_order!

      run_purchasable_callbacks(:after_purchase)
    end
  rescue => e
    Effective::Order.transaction do
      save!(validate: false)
      update_purchasables_purchased_order!
    end

    raise(e)
  end

  send_order_receipts! if email
  after_commit { sync_quickbooks!(skip: skip_quickbooks) }

  true
end

#purchased?(provider = nil) ⇒ Boolean

Returns:

  • (Boolean)


435
436
437
438
439
# File 'app/models/effective/order.rb', line 435

def purchased?(provider = nil)
  return false if (state != EffectiveOrders::PURCHASED)
  return true if provider.nil? || payment_provider == provider.to_s
  false
end

#purchased_with_credit_card? ⇒ Boolean

Returns:

  • (Boolean)


445
446
447
# File 'app/models/effective/order.rb', line 445

def purchased_with_credit_card?
  purchased? && EffectiveOrders.credit_card_payment_providers.include?(payment_provider)
end

#purchased_without_credit_card? ⇒ Boolean

Returns:

  • (Boolean)


449
450
451
# File 'app/models/effective/order.rb', line 449

def purchased_without_credit_card?
  purchased? && EffectiveOrders.credit_card_payment_providers.exclude?(payment_provider)
end

#refund? ⇒ Boolean

Returns:

  • (Boolean)


509
510
511
# File 'app/models/effective/order.rb', line 509

def refund?
  total.to_i < 0
end

#remove(*items) ⇒ Object



239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
# File 'app/models/effective/order.rb', line 239

def remove(*items)
  raise 'unable to alter a purchased order' if purchased?
  raise 'unable to alter a declined order' if declined?

  removed = items.map do |item|
    order_item = if item.kind_of?(Effective::OrderItem)
      order_items.find { |oi| oi == item }
    else
      order_items.find { |oi| oi.purchasable == item }
    end

    raise("Unable to find order item for #{item}") if order_item.blank?
    order_item
  end

  removed.each { |order_item| order_item.mark_for_destruction }

  # Make sure to reset stored aggregates
  assign_attributes(subtotal: nil, tax_rate: nil, tax: nil, amount_owing: nil, surcharge_percent: nil, surcharge: nil, total: nil)

  removed.length == 1 ? removed.first : removed
end

#reportable_scopes ⇒ Object

effective_reports



100
101
102
# File 'app/models/effective/order.rb', line 100

def reportable_scopes
  { purchased: nil, not_purchased: nil, deferred: nil, refunds: nil, pending_refunds: nil }
end

#send_mark_as_paid_email_to_buyer? ⇒ Boolean

Returns:

  • (Boolean)


546
547
548
# File 'app/models/effective/order.rb', line 546

def send_mark_as_paid_email_to_buyer?
  EffectiveResources.truthy?(send_mark_as_paid_email_to_buyer)
end

#send_order_receipt_to_admin! ⇒ Object



729
730
731
# File 'app/models/effective/order.rb', line 729

def send_order_receipt_to_admin!
  EffectiveOrders.send_email(:order_receipt_to_admin, self) if purchased?
end

#send_order_receipt_to_admin? ⇒ Boolean

Returns:

  • (Boolean)


524
525
526
527
# File 'app/models/effective/order.rb', line 524

def send_order_receipt_to_admin?
  return false if free? && !EffectiveOrders.send_order_receipts_when_free
  EffectiveOrders.send_order_receipt_to_admin
end

#send_order_receipt_to_buyer! ⇒ Object Also known as: send_buyer_receipt!



733
734
735
# File 'app/models/effective/order.rb', line 733

def send_order_receipt_to_buyer!
  EffectiveOrders.send_email(:order_receipt_to_buyer, self) if purchased?
end

#send_order_receipt_to_buyer? ⇒ Boolean

Returns:

  • (Boolean)


529
530
531
532
# File 'app/models/effective/order.rb', line 529

def send_order_receipt_to_buyer?
  return false if free? && !EffectiveOrders.send_order_receipts_when_free
  EffectiveOrders.send_order_receipt_to_buyer
end

#send_order_receipts! ⇒ Object



723
724
725
726
727
# File 'app/models/effective/order.rb', line 723

def send_order_receipts!
  send_order_receipt_to_admin! if send_order_receipt_to_admin?
  send_order_receipt_to_buyer! if send_order_receipt_to_buyer?
  send_refund_notification! if send_refund_notification_to_admin?
end

#send_payment_request_to_buyer! ⇒ Object



738
739
740
# File 'app/models/effective/order.rb', line 738

def send_payment_request_to_buyer!
  EffectiveOrders.send_email(:payment_request_to_buyer, self) unless purchased?
end

#send_payment_request_to_buyer? ⇒ Boolean

Returns:

  • (Boolean)


534
535
536
537
538
539
# File 'app/models/effective/order.rb', line 534

def send_payment_request_to_buyer?
  return false if free? && !EffectiveOrders.send_order_receipts_when_free
  return false if refund?

  EffectiveResources.truthy?(send_payment_request_to_buyer)
end

#send_pending_order_invoice_to_buyer! ⇒ Object



742
743
744
# File 'app/models/effective/order.rb', line 742

def send_pending_order_invoice_to_buyer!
  EffectiveOrders.send_email(:pending_order_invoice_to_buyer, self) unless purchased?
end

#send_refund_notification! ⇒ Object



746
747
748
# File 'app/models/effective/order.rb', line 746

def send_refund_notification!
  EffectiveOrders.send_email(:refund_notification_to_admin, self) if refund?
end

#send_refund_notification_to_admin? ⇒ Boolean

Returns:

  • (Boolean)


541
542
543
544
# File 'app/models/effective/order.rb', line 541

def send_refund_notification_to_admin?
  return false unless refund?
  EffectiveOrders.send_refund_notification_to_admin
end

#skip_buyer_validations? ⇒ Boolean

Returns:

  • (Boolean)


550
551
552
# File 'app/models/effective/order.rb', line 550

def skip_buyer_validations?
  EffectiveResources.truthy?(skip_buyer_validations)
end

#skip_quickbooks! ⇒ Object



664
665
666
# File 'app/models/effective/order.rb', line 664

def skip_quickbooks!
  sync_quickbooks!(skip: true)
end

#subtotal ⇒ Object



465
466
467
# File 'app/models/effective/order.rb', line 465

def subtotal
  self[:subtotal] || get_subtotal()
end

#surcharge ⇒ Object



485
486
487
# File 'app/models/effective/order.rb', line 485

def surcharge
  self[:surcharge] || get_surcharge()
end

#surcharge_percent ⇒ Object



481
482
483
# File 'app/models/effective/order.rb', line 481

def surcharge_percent
  self[:surcharge_percent] || get_surcharge_percent()
end

#surcharge_tax ⇒ Object



489
490
491
# File 'app/models/effective/order.rb', line 489

def surcharge_tax
  self[:surcharge_tax] || get_surcharge_tax()
end

#sync_quickbooks!(skip:) ⇒ Object

We support two different Quickbooks synchronization gems: effective_qb_sync and effective_qb_online



652
653
654
655
656
657
658
659
660
661
662
# File 'app/models/effective/order.rb', line 652

def sync_quickbooks!(skip:)
  if EffectiveOrders.qb_online?
    skip ? EffectiveQbOnline.skip_order!(self) : EffectiveQbOnline.sync_order!(self)
  end

  if EffectiveOrders.qb_sync?
    skip ? EffectiveQbSync.skip_order!(self) : true # Nothing to do
  end

  true
end

#tax ⇒ Object



473
474
475
# File 'app/models/effective/order.rb', line 473

def tax
  self[:tax] || get_tax()
end

#tax_rate ⇒ Object



469
470
471
# File 'app/models/effective/order.rb', line 469

def tax_rate
  self[:tax_rate] || get_tax_rate()
end

#to_s ⇒ Object



332
333
334
# File 'app/models/effective/order.rb', line 332

def to_s
  [label, ' #', to_param].join
end

#total ⇒ Object



493
494
495
# File 'app/models/effective/order.rb', line 493

def total
  self[:total] || get_total()
end

#total_label ⇒ Object



350
351
352
# File 'app/models/effective/order.rb', line 350

def total_label
  purchased? ? 'Total Paid' : 'Total Due'
end

#total_with_surcharge ⇒ Object



497
498
499
# File 'app/models/effective/order.rb', line 497

def total_with_surcharge
  get_total_with_surcharge()
end

#total_without_surcharge ⇒ Object



501
502
503
# File 'app/models/effective/order.rb', line 501

def total_without_surcharge
  get_total_without_surcharge()
end

#transaction_id ⇒ Object

For moneris and moneris_checkout. Just a unique value. Must be 50 characters or fewer or will raise moneris error.



397
398
399
# File 'app/models/effective/order.rb', line 397

def transaction_id
  [to_param, billing_name.to_s.parameterize.first(20).presence, Time.zone.now.to_i, rand(1000..9999)].compact.join('-')
end

#update_prices! ⇒ Object



315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
# File 'app/models/effective/order.rb', line 315

def update_prices!
  raise('already purchased') if purchased?
  raise('must be pending or confirmed') unless pending? || confirmed?

  present_order_items.each do |item|
    purchasable = item.purchasable

    if purchasable.blank? || purchasable.marked_for_destruction?
      item.mark_for_destruction
    else
      item.assign_purchasable_attributes
    end
  end

  save!
end

#update_purchasable_attributes ⇒ Object

Called by effective_memberships to update prices from purchasable fees Not called internally



592
593
594
# File 'app/models/effective/order.rb', line 592

def update_purchasable_attributes
  present_order_items.each { |oi| oi.update_purchasable_attributes }
end

#was_purchased? ⇒ Boolean

Returns:

  • (Boolean)


441
442
443
# File 'app/models/effective/order.rb', line 441

def was_purchased?
  state_was == EffectiveOrders::PURCHASED
end