Class: Effective::Order
- Inherits:
-
ActiveRecord::Base
- Object
- ActiveRecord::Base
- Effective::Order
- Defined in:
- app/models/effective/order.rb
Instance Attribute Summary collapse
-
#confirmed_checkout ⇒ Object
Set on the Checkout Step 1.
-
#send_mark_as_paid_email_to_buyer ⇒ Object
Set by Admin::Orders#mark_as_paid.
-
#send_payment_request_to_buyer ⇒ Object
Settings in the /admin action forms.
-
#skip_buyer_validations ⇒ Object
Set by Admin::Orders#create.
-
#terms_and_conditions ⇒ Object
Yes, I agree to the terms and conditions.
Instance Method Summary collapse
- #abandoned? ⇒ Boolean
-
#add(*items, quantity: 1) ⇒ Object
Items can be an Effective::Cart, an Effective::order, a single acts_as_purchasable, or multiple acts_as_purchasables add(Product.first) => returns an Effective::OrderItem add(Product.first, current_cart) => returns an array of Effective::OrderItems.
- #amount_owing ⇒ Object
-
#assign_confirmed_if_valid! ⇒ Object
This lets us skip to the confirmed workflow for an admin…
- #billing_first_name ⇒ Object
- #billing_last_name ⇒ Object
-
#confirm! ⇒ Object
Used by admin checkout only.
- #confirmed? ⇒ Boolean
-
#custom_order? ⇒ Boolean
A custom order is one that was created by an admin We allow custom orders to have their order items updated.
- #decline!(payment: 'none', provider: 'none', card: 'none', validate: true) ⇒ Object
- #declined? ⇒ Boolean
- #defer!(provider: 'none', email: true) ⇒ Object
- #deferred? ⇒ Boolean
- #done? ⇒ Boolean
- #duplicate ⇒ Object
-
#emails_send_to ⇒ Object
Doesn’t control anything.
- #free? ⇒ Boolean
- #in_progress? ⇒ Boolean
-
#initialize(atts = nil, &block) ⇒ Order
constructor
Effective::Order.new(items: Product.first) Effective::Order.new(items: [Product.first, Product.second], user: User.first) Effective::Order.new(items: Product.first, user: User.first, billing_address: Effective::Address.new, shipping_address: Effective::Address.new).
- #label ⇒ Object
-
#mark_as_purchased! ⇒ Object
Call this as a way to skip over non consequential orders And mark some purchasables purchased This is different than the Mark as Paid payment processor.
- #num_items ⇒ Object
-
#payment_method ⇒ Object
Visa - 1234.
-
#pending! ⇒ Object
This is called from admin/orders#create This is intended for use as an admin action only It skips any address or bad user validations It’s basically the same as save! on a new order, except it might send the payment request to buyer.
- #pending? ⇒ Boolean
- #pending_refund? ⇒ Boolean
- #purchasables ⇒ Object
-
#purchase!(payment: nil, provider: nil, card: nil, email: true, skip_buyer_validations: false, skip_quickbooks: false) ⇒ Object
Effective::Order.new(items: Product.first, user: User.first).purchase!(email: false).
- #purchased?(provider = nil) ⇒ Boolean
- #purchased_with_credit_card? ⇒ Boolean
- #purchased_without_credit_card? ⇒ Boolean
- #refund? ⇒ Boolean
- #remove(*items) ⇒ Object
-
#reportable_scopes ⇒ Object
effective_reports.
- #send_mark_as_paid_email_to_buyer? ⇒ Boolean
- #send_order_receipt_to_admin! ⇒ Object
- #send_order_receipt_to_admin? ⇒ Boolean
- #send_order_receipt_to_buyer! ⇒ Object (also: #send_buyer_receipt!)
- #send_order_receipt_to_buyer? ⇒ Boolean
- #send_order_receipts! ⇒ Object
- #send_payment_request_to_buyer! ⇒ Object
- #send_payment_request_to_buyer? ⇒ Boolean
- #send_pending_order_invoice_to_buyer! ⇒ Object
- #send_refund_notification! ⇒ Object
- #send_refund_notification_to_admin? ⇒ Boolean
- #skip_buyer_validations? ⇒ Boolean
- #skip_quickbooks! ⇒ Object
- #subtotal ⇒ Object
- #surcharge ⇒ Object
- #surcharge_percent ⇒ Object
- #surcharge_tax ⇒ Object
-
#sync_quickbooks!(skip:) ⇒ Object
We support two different Quickbooks synchronization gems: effective_qb_sync and effective_qb_online.
- #tax ⇒ Object
- #tax_rate ⇒ Object
- #to_s ⇒ Object
- #total ⇒ Object
- #total_label ⇒ Object
- #total_with_surcharge ⇒ Object
- #total_without_surcharge ⇒ Object
-
#transaction_id ⇒ Object
For moneris and moneris_checkout.
- #update_prices! ⇒ Object
- #was_purchased? ⇒ Boolean
Constructor Details
#initialize(atts = nil, &block) ⇒ Order
Effective::Order.new(items: Product.first) Effective::Order.new(items: [Product.first, Product.second], user: User.first) Effective::Order.new(items: Product.first, user: User.first, billing_address: Effective::Address.new, shipping_address: Effective::Address.new)
187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 |
# File 'app/models/effective/order.rb', line 187 def initialize(atts = nil, &block) super(state: EffectiveOrders::PENDING) # Initialize with state: PENDING return self unless atts.present? if atts.kind_of?(Hash) items = Array(atts[:item]) + Array(atts[:items]) self.user = atts[:user] || items.first.try(:user) if (address = atts[:billing_address]).present? self.billing_address = address self.billing_address.full_name ||= user.to_s.presence end if (address = atts[:shipping_address]).present? self.shipping_address = address self.shipping_address.full_name ||= user.to_s.presence end atts.except(:item, :items, :user, :billing_address, :shipping_address).each do |key, value| self.send("#{key}=", value) end add(items) if items.present? else # Attributes are not a Hash self.user = atts.user if atts.respond_to?(:user) add(atts) end self end |
Instance Attribute Details
#confirmed_checkout ⇒ Object
Set on the Checkout Step 1
24 25 26 |
# File 'app/models/effective/order.rb', line 24 def confirmed_checkout @confirmed_checkout end |
#send_mark_as_paid_email_to_buyer ⇒ Object
Set by Admin::Orders#mark_as_paid
28 29 30 |
# File 'app/models/effective/order.rb', line 28 def send_mark_as_paid_email_to_buyer @send_mark_as_paid_email_to_buyer end |
#send_payment_request_to_buyer ⇒ Object
Settings in the /admin action forms
27 28 29 |
# File 'app/models/effective/order.rb', line 27 def send_payment_request_to_buyer @send_payment_request_to_buyer end |
#skip_buyer_validations ⇒ Object
Set by Admin::Orders#create
29 30 31 |
# File 'app/models/effective/order.rb', line 29 def skip_buyer_validations @skip_buyer_validations end |
#terms_and_conditions ⇒ Object
Yes, I agree to the terms and conditions
23 24 25 |
# File 'app/models/effective/order.rb', line 23 def terms_and_conditions @terms_and_conditions end |
Instance Method Details
#abandoned? ⇒ Boolean
438 439 440 |
# File 'app/models/effective/order.rb', line 438 def abandoned? state == EffectiveOrders::ABANDONED end |
#add(*items, quantity: 1) ⇒ Object
Items can be an Effective::Cart, an Effective::order, a single acts_as_purchasable, or multiple acts_as_purchasables add(Product.first) => returns an Effective::OrderItem add(Product.first, current_cart) => returns an array of Effective::OrderItems
246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 |
# File 'app/models/effective/order.rb', line 246 def add(*items, quantity: 1) raise 'unable to alter a purchased order' if purchased? raise 'unable to alter a declined order' if declined? cart_items = items.flatten.flat_map do |item| if item.kind_of?(Effective::Cart) item.cart_items.to_a elsif item.kind_of?(ActsAsPurchasable) Effective::CartItem.new(quantity: quantity, purchasable: item) elsif item.kind_of?(Effective::Order) # Duplicate an existing order self.note_to_buyer ||= item.note_to_buyer self.note_internal ||= item.note_internal self.cc ||= item.cc item.order_items.select { |oi| oi.purchasable.kind_of?(Effective::Product) }.map do |oi| purchasable = oi.purchasable product = Effective::Product.new(name: purchasable.purchasable_name, price: purchasable.price, tax_exempt: purchasable.tax_exempt) # Copy over any extended attributes that may have been created atts = purchasable.dup.attributes.except('name', 'price', 'tax_exempt', 'purchased_order_id').compact atts.each do |k, v| next unless product.respond_to?("#{k}=") && product.respond_to?(k) product.send("#{k}=", v) if product.send(k).blank? end Effective::CartItem.new(quantity: oi.quantity, purchasable: product) end else raise 'add() expects one or more acts_as_purchasable objects, or an Effective::Cart' end end.compact # Make sure to reset stored aggregates assign_attributes(subtotal: nil, tax_rate: nil, tax: nil, amount_owing: nil, surcharge_percent: nil, surcharge: nil, total: nil) retval = cart_items.map do |item| order_items.build( name: item.name, quantity: item.quantity, price: item.price, tax_exempt: (item.tax_exempt || false), ).tap { |order_item| order_item.purchasable = item.purchasable } end retval.size == 1 ? retval.first : retval end |
#amount_owing ⇒ Object
458 459 460 |
# File 'app/models/effective/order.rb', line 458 def amount_owing self[:amount_owing] || get_amount_owing() end |
#assign_confirmed_if_valid! ⇒ Object
This lets us skip to the confirmed workflow for an admin…
560 561 562 563 564 565 566 567 568 569 |
# File 'app/models/effective/order.rb', line 560 def assign_confirmed_if_valid! return unless pending? self.state = EffectiveOrders::CONFIRMED return true if valid? self.errors.clear self.state = EffectiveOrders::PENDING false end |
#billing_first_name ⇒ Object
382 383 384 |
# File 'app/models/effective/order.rb', line 382 def billing_first_name billing_name.to_s.split(' ').first end |
#billing_last_name ⇒ Object
386 387 388 |
# File 'app/models/effective/order.rb', line 386 def billing_last_name Array(billing_name.to_s.split(' ')[1..-1]).join(' ') end |
#confirm! ⇒ Object
Used by admin checkout only
554 555 556 557 |
# File 'app/models/effective/order.rb', line 554 def confirm! return false if purchased? update!(state: EffectiveOrders::CONFIRMED) end |
#confirmed? ⇒ Boolean
394 395 396 |
# File 'app/models/effective/order.rb', line 394 def confirmed? state == EffectiveOrders::CONFIRMED end |
#custom_order? ⇒ Boolean
A custom order is one that was created by an admin We allow custom orders to have their order items updated
412 413 414 |
# File 'app/models/effective/order.rb', line 412 def custom_order? order_items.all? { |oi| oi.purchasable_type == 'Effective::Product' } end |
#decline!(payment: 'none', provider: 'none', card: 'none', validate: true) ⇒ Object
649 650 651 652 653 654 655 656 657 658 659 660 661 662 663 664 665 666 667 668 669 670 671 672 673 674 675 676 677 678 679 680 681 |
# File 'app/models/effective/order.rb', line 649 def decline!(payment: 'none', provider: 'none', card: 'none', validate: true) return false if declined? raise('order already purchased') if purchased? error = nil assign_attributes( state: EffectiveOrders::DECLINED, purchased_at: nil, payment: payment_to_h(payment), payment_provider: provider, payment_card: (card.presence || 'none'), skip_buyer_validations: true ) Effective::Order.transaction do begin run_purchasable_callbacks(:before_decline) save!(validate: validate) run_purchasable_callbacks(:after_decline) rescue => e self.state = state_was error = e. raise ::ActiveRecord::Rollback end end raise "Failed to decline order: #{error || errors.full_messages.to_sentence}" unless error.nil? true end |
#declined? ⇒ Boolean
434 435 436 |
# File 'app/models/effective/order.rb', line 434 def declined? state == EffectiveOrders::DECLINED end |
#defer!(provider: 'none', email: true) ⇒ Object
638 639 640 641 642 643 644 645 646 647 |
# File 'app/models/effective/order.rb', line 638 def defer!(provider: 'none', email: true) return false if purchased? assign_attributes(state: EffectiveOrders::DEFERRED, payment_provider: provider) save! send_payment_request_to_buyer! if email true end |
#deferred? ⇒ Boolean
398 399 400 |
# File 'app/models/effective/order.rb', line 398 def deferred? state == EffectiveOrders::DEFERRED end |
#done? ⇒ Boolean
406 407 408 |
# File 'app/models/effective/order.rb', line 406 def done? persisted? && (purchased? || declined?) end |
#duplicate ⇒ Object
373 374 375 |
# File 'app/models/effective/order.rb', line 373 def duplicate Effective::Order.new(self) end |
#emails_send_to ⇒ Object
Doesn’t control anything. Purely for the flash messaging
684 685 686 |
# File 'app/models/effective/order.rb', line 684 def emails_send_to [email, cc.presence].compact.to_sentence end |
#free? ⇒ Boolean
486 487 488 |
# File 'app/models/effective/order.rb', line 486 def free? total == 0 end |
#in_progress? ⇒ Boolean
402 403 404 |
# File 'app/models/effective/order.rb', line 402 def in_progress? pending? || confirmed? || deferred? end |
#label ⇒ Object
317 318 319 320 321 322 323 324 325 326 327 328 329 |
# File 'app/models/effective/order.rb', line 317 def label if refund? && purchased? 'Refund' elsif purchased? 'Receipt' elsif refund? && (pending? || confirmed?) 'Pending Refund' elsif (pending? || confirmed?) 'Pending Order' else 'Order' end end |
#mark_as_purchased! ⇒ Object
Call this as a way to skip over non consequential orders And mark some purchasables purchased This is different than the Mark as Paid payment processor
574 575 576 |
# File 'app/models/effective/order.rb', line 574 def mark_as_purchased! purchase!(skip_buyer_validations: true, email: false, skip_quickbooks: true) end |
#num_items ⇒ Object
501 502 503 |
# File 'app/models/effective/order.rb', line 501 def num_items present_order_items.map { |oi| oi.quantity }.sum end |
#payment_method ⇒ Object
Visa - 1234
336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 |
# File 'app/models/effective/order.rb', line 336 def payment_method return nil unless purchased? provider = payment_provider if ['cheque', 'etransfer', 'phone', 'credit card'].include?(payment_provider) # Normalize payment card card = case payment_card.to_s.downcase.gsub(' ', '').strip when '' then nil when 'v', 'visa' then 'Visa' when 'm', 'mc', 'master', 'mastercard' then 'MasterCard' when 'a', 'ax', 'american', 'americanexpress' then 'American Express' when 'd', 'discover' then 'Discover' else payment_card.to_s end # Try again if card == 'none' && payment['card_type'].present? card = case payment['card_type'].to_s.downcase.gsub(' ', '').strip when '' then nil when 'v', 'visa' then 'Visa' when 'm', 'mc', 'master', 'mastercard' then 'MasterCard' when 'a', 'ax', 'american', 'americanexpress' then 'American Express' when 'd', 'discover' then 'Discover' else payment_card.to_s end end last4 = if payment[:active_card] && payment[:active_card].include?('**** **** ****') payment[:active_card][15,4] end # stripe, moneris, moneris_checkout last4 ||= (payment['f4l4'] || payment['first6last4']).to_s.last(4) [provider.presence, card.presence, last4.presence].compact.join(' - ') end |
#pending! ⇒ Object
This is called from admin/orders#create This is intended for use as an admin action only It skips any address or bad user validations It’s basically the same as save! on a new order, except it might send the payment request to buyer
539 540 541 542 543 544 545 546 547 548 549 550 551 |
# File 'app/models/effective/order.rb', line 539 def pending! return false if purchased? self.state = EffectiveOrders::PENDING self.addresses.clear if addresses.any? { |address| address.valid? == false } save! if send_payment_request_to_buyer? after_commit { send_payment_request_to_buyer! } end true end |
#pending? ⇒ Boolean
390 391 392 |
# File 'app/models/effective/order.rb', line 390 def pending? state == EffectiveOrders::PENDING end |
#pending_refund? ⇒ Boolean
494 495 496 497 498 499 |
# File 'app/models/effective/order.rb', line 494 def pending_refund? return false if EffectiveOrders.buyer_purchases_refund? return false if purchased? refund? end |
#purchasables ⇒ Object
442 443 444 |
# File 'app/models/effective/order.rb', line 442 def purchasables present_order_items.map { |order_item| order_item.purchasable } end |
#purchase!(payment: nil, provider: nil, card: nil, email: true, skip_buyer_validations: false, skip_quickbooks: false) ⇒ Object
Effective::Order.new(items: Product.first, user: User.first).purchase!(email: false)
579 580 581 582 583 584 585 586 587 588 589 590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 |
# File 'app/models/effective/order.rb', line 579 def purchase!(payment: nil, provider: nil, card: nil, email: true, skip_buyer_validations: false, skip_quickbooks: false) return true if purchased? # Assign attributes assign_attributes( state: EffectiveOrders::PURCHASED, skip_buyer_validations: skip_buyer_validations, payment: payment_to_h(payment.presence || 'none'), purchased_at: (purchased_at.presence || Time.zone.now), payment_provider: (provider.presence || 'none'), payment_card: (card.presence || 'none') ) # Updates surcharge and total based on payment_provider assign_order_charges() begin Effective::Order.transaction do run_purchasable_callbacks(:before_purchase) save! update_purchasables_purchased_order! run_purchasable_callbacks(:after_purchase) end rescue => e Effective::Order.transaction do save!(validate: false) update_purchasables_purchased_order! end raise(e) end send_order_receipts! if email after_commit { sync_quickbooks!(skip: skip_quickbooks) } true end |
#purchased?(provider = nil) ⇒ Boolean
416 417 418 419 420 |
# File 'app/models/effective/order.rb', line 416 def purchased?(provider = nil) return false if (state != EffectiveOrders::PURCHASED) return true if provider.nil? || payment_provider == provider.to_s false end |
#purchased_with_credit_card? ⇒ Boolean
426 427 428 |
# File 'app/models/effective/order.rb', line 426 def purchased_with_credit_card? purchased? && EffectiveOrders.credit_card_payment_providers.include?(payment_provider) end |
#purchased_without_credit_card? ⇒ Boolean
430 431 432 |
# File 'app/models/effective/order.rb', line 430 def purchased_without_credit_card? purchased? && EffectiveOrders.credit_card_payment_providers.exclude?(payment_provider) end |
#refund? ⇒ Boolean
490 491 492 |
# File 'app/models/effective/order.rb', line 490 def refund? total.to_i < 0 end |
#remove(*items) ⇒ Object
220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 |
# File 'app/models/effective/order.rb', line 220 def remove(*items) raise 'unable to alter a purchased order' if purchased? raise 'unable to alter a declined order' if declined? removed = items.map do |item| order_item = if item.kind_of?(Effective::OrderItem) order_items.find { |oi| oi == item } else order_items.find { |oi| oi.purchasable == item } end raise("Unable to find order item for #{item}") if order_item.blank? order_item end removed.each { |order_item| order_item.mark_for_destruction } # Make sure to reset stored aggregates assign_attributes(subtotal: nil, tax_rate: nil, tax: nil, amount_owing: nil, surcharge_percent: nil, surcharge: nil, total: nil) removed.length == 1 ? removed.first : removed end |
#reportable_scopes ⇒ Object
effective_reports
92 93 94 |
# File 'app/models/effective/order.rb', line 92 def reportable_scopes { purchased: nil, not_purchased: nil, deferred: nil, refunds: nil, pending_refunds: nil } end |
#send_mark_as_paid_email_to_buyer? ⇒ Boolean
527 528 529 |
# File 'app/models/effective/order.rb', line 527 def send_mark_as_paid_email_to_buyer? EffectiveResources.truthy?(send_mark_as_paid_email_to_buyer) end |
#send_order_receipt_to_admin! ⇒ Object
694 695 696 |
# File 'app/models/effective/order.rb', line 694 def send_order_receipt_to_admin! EffectiveOrders.send_email(:order_receipt_to_admin, self) if purchased? end |
#send_order_receipt_to_admin? ⇒ Boolean
505 506 507 508 |
# File 'app/models/effective/order.rb', line 505 def send_order_receipt_to_admin? return false if free? && !EffectiveOrders.send_order_receipts_when_free EffectiveOrders.send_order_receipt_to_admin end |
#send_order_receipt_to_buyer! ⇒ Object Also known as: send_buyer_receipt!
698 699 700 |
# File 'app/models/effective/order.rb', line 698 def send_order_receipt_to_buyer! EffectiveOrders.send_email(:order_receipt_to_buyer, self) if purchased? end |
#send_order_receipt_to_buyer? ⇒ Boolean
510 511 512 513 |
# File 'app/models/effective/order.rb', line 510 def send_order_receipt_to_buyer? return false if free? && !EffectiveOrders.send_order_receipts_when_free EffectiveOrders.send_order_receipt_to_buyer end |
#send_order_receipts! ⇒ Object
688 689 690 691 692 |
# File 'app/models/effective/order.rb', line 688 def send_order_receipts! send_order_receipt_to_admin! if send_order_receipt_to_admin? send_order_receipt_to_buyer! if send_order_receipt_to_buyer? send_refund_notification! if send_refund_notification_to_admin? end |
#send_payment_request_to_buyer! ⇒ Object
703 704 705 |
# File 'app/models/effective/order.rb', line 703 def send_payment_request_to_buyer! EffectiveOrders.send_email(:payment_request_to_buyer, self) unless purchased? end |
#send_payment_request_to_buyer? ⇒ Boolean
515 516 517 518 519 520 |
# File 'app/models/effective/order.rb', line 515 def send_payment_request_to_buyer? return false if free? && !EffectiveOrders.send_order_receipts_when_free return false if refund? EffectiveResources.truthy?(send_payment_request_to_buyer) end |
#send_pending_order_invoice_to_buyer! ⇒ Object
707 708 709 |
# File 'app/models/effective/order.rb', line 707 def send_pending_order_invoice_to_buyer! EffectiveOrders.send_email(:pending_order_invoice_to_buyer, self) unless purchased? end |
#send_refund_notification! ⇒ Object
711 712 713 |
# File 'app/models/effective/order.rb', line 711 def send_refund_notification! EffectiveOrders.send_email(:refund_notification_to_admin, self) if refund? end |
#send_refund_notification_to_admin? ⇒ Boolean
522 523 524 525 |
# File 'app/models/effective/order.rb', line 522 def send_refund_notification_to_admin? return false unless refund? EffectiveOrders.send_refund_notification_to_admin end |
#skip_buyer_validations? ⇒ Boolean
531 532 533 |
# File 'app/models/effective/order.rb', line 531 def skip_buyer_validations? EffectiveResources.truthy?(skip_buyer_validations) end |
#skip_quickbooks! ⇒ Object
634 635 636 |
# File 'app/models/effective/order.rb', line 634 def skip_quickbooks! sync_quickbooks!(skip: true) end |
#subtotal ⇒ Object
446 447 448 |
# File 'app/models/effective/order.rb', line 446 def subtotal self[:subtotal] || get_subtotal() end |
#surcharge ⇒ Object
466 467 468 |
# File 'app/models/effective/order.rb', line 466 def surcharge self[:surcharge] || get_surcharge() end |
#surcharge_percent ⇒ Object
462 463 464 |
# File 'app/models/effective/order.rb', line 462 def surcharge_percent self[:surcharge_percent] || get_surcharge_percent() end |
#surcharge_tax ⇒ Object
470 471 472 |
# File 'app/models/effective/order.rb', line 470 def surcharge_tax self[:surcharge_tax] || get_surcharge_tax() end |
#sync_quickbooks!(skip:) ⇒ Object
We support two different Quickbooks synchronization gems: effective_qb_sync and effective_qb_online
622 623 624 625 626 627 628 629 630 631 632 |
# File 'app/models/effective/order.rb', line 622 def sync_quickbooks!(skip:) if EffectiveOrders.qb_online? skip ? EffectiveQbOnline.skip_order!(self) : EffectiveQbOnline.sync_order!(self) end if EffectiveOrders.qb_sync? skip ? EffectiveQbSync.skip_order!(self) : true # Nothing to do end true end |
#tax ⇒ Object
454 455 456 |
# File 'app/models/effective/order.rb', line 454 def tax self[:tax] || get_tax() end |
#tax_rate ⇒ Object
450 451 452 |
# File 'app/models/effective/order.rb', line 450 def tax_rate self[:tax_rate] || get_tax_rate() end |
#to_s ⇒ Object
313 314 315 |
# File 'app/models/effective/order.rb', line 313 def to_s [label, ' #', to_param].join end |
#total ⇒ Object
474 475 476 |
# File 'app/models/effective/order.rb', line 474 def total self[:total] || get_total() end |
#total_label ⇒ Object
331 332 333 |
# File 'app/models/effective/order.rb', line 331 def total_label purchased? ? 'Total Paid' : 'Total Due' end |
#total_with_surcharge ⇒ Object
478 479 480 |
# File 'app/models/effective/order.rb', line 478 def total_with_surcharge get_total_with_surcharge() end |
#total_without_surcharge ⇒ Object
482 483 484 |
# File 'app/models/effective/order.rb', line 482 def total_without_surcharge get_total_without_surcharge() end |
#transaction_id ⇒ Object
For moneris and moneris_checkout. Just a unique value. Must be 50 characters or fewer or will raise moneris error.
378 379 380 |
# File 'app/models/effective/order.rb', line 378 def transaction_id [to_param, billing_name.to_s.parameterize.first(20).presence, Time.zone.now.to_i, rand(1000..9999)].compact.join('-') end |
#update_prices! ⇒ Object
296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 |
# File 'app/models/effective/order.rb', line 296 def update_prices! raise('already purchased') if purchased? raise('must be pending or confirmed') unless pending? || confirmed? present_order_items.each do |item| purchasable = item.purchasable if purchasable.blank? || purchasable.marked_for_destruction? item.mark_for_destruction else item.assign_purchasable_attributes end end save! end |
#was_purchased? ⇒ Boolean
422 423 424 |
# File 'app/models/effective/order.rb', line 422 def was_purchased? state_was == EffectiveOrders::PURCHASED end |