Class: Effective::Order
- Inherits:
-
ActiveRecord::Base
- Object
- ActiveRecord::Base
- Effective::Order
- Defined in:
- app/models/effective/order.rb
Instance Attribute Summary collapse
-
#confirmed_checkout ⇒ Object
Set on the Checkout Step 1.
-
#send_mark_as_paid_email_to_buyer ⇒ Object
Set by Admin::Orders#mark_as_paid.
-
#send_payment_request_to_buyer ⇒ Object
Settings in the /admin action forms.
-
#skip_buyer_validations ⇒ Object
Set by Admin::Orders#create.
-
#terms_and_conditions ⇒ Object
Yes, I agree to the terms and conditions.
Instance Method Summary collapse
- #abandoned? ⇒ Boolean
-
#add(*items, quantity: 1) ⇒ Object
Items can be an Effective::Cart, an Effective::order, a single acts_as_purchasable, or multiple acts_as_purchasables add(Product.first) => returns an Effective::OrderItem add(Product.first, current_cart) => returns an array of Effective::OrderItems.
-
#assign_confirmed_if_valid! ⇒ Object
This lets us skip to the confirmed workflow for an admin...
- #billing_first_name ⇒ Object
- #billing_last_name ⇒ Object
-
#confirm! ⇒ Object
Used by admin checkout only.
- #confirmed? ⇒ Boolean
-
#custom_order? ⇒ Boolean
A custom order is one that was created by an admin We allow custom orders to have their order items updated.
- #decline!(payment: 'none', provider: 'none', card: 'none', validate: true) ⇒ Object
- #declined? ⇒ Boolean
- #defer!(provider: 'none', email: true) ⇒ Object
- #deferred? ⇒ Boolean
- #done? ⇒ Boolean
- #duplicate ⇒ Object
-
#emails_send_to ⇒ Object
Doesn't control anything.
- #free? ⇒ Boolean
- #in_progress? ⇒ Boolean
-
#initialize(atts = nil, &block) ⇒ Order
constructor
Effective::Order.new(items: Product.first) Effective::Order.new(items: [Product.first, Product.second], user: User.first) Effective::Order.new(items: Product.first, user: User.first, billing_address: Effective::Address.new, shipping_address: Effective::Address.new).
- #label ⇒ Object
-
#mark_as_purchased! ⇒ Object
Call this as a way to skip over non consequential orders And mark some purchasables purchased.
- #num_items ⇒ Object
-
#payment_method ⇒ Object
Visa - 1234.
-
#pending! ⇒ Object
This is called from admin/orders#create This is intended for use as an admin action only It skips any address or bad user validations It's basically the same as save! on a new order, except it might send the payment request to buyer.
- #pending? ⇒ Boolean
- #purchasables ⇒ Object
-
#purchase!(payment: 'none', provider: 'none', card: 'none', email: true, skip_buyer_validations: false, skip_quickbooks: false) ⇒ Object
Effective::Order.new(items: Product.first, user: User.first).purchase!(email: false).
- #purchased?(provider = nil) ⇒ Boolean
- #refund? ⇒ Boolean
- #remove(*items) ⇒ Object
- #send_mark_as_paid_email_to_buyer? ⇒ Boolean
- #send_order_receipt_to_admin! ⇒ Object
- #send_order_receipt_to_admin? ⇒ Boolean
- #send_order_receipt_to_buyer! ⇒ Object
- #send_order_receipt_to_buyer? ⇒ Boolean
- #send_order_receipts! ⇒ Object
- #send_payment_request_to_buyer! ⇒ Object
- #send_payment_request_to_buyer? ⇒ Boolean
- #send_pending_order_invoice_to_buyer! ⇒ Object
- #send_refund_notification! ⇒ Object
- #skip_buyer_validations? ⇒ Boolean
- #skip_quickbooks! ⇒ Object
- #subtotal ⇒ Object
-
#sync_quickbooks!(skip:) ⇒ Object
We support two different Quickbooks synchronization gems: effective_qb_sync and effective_qb_online.
- #tax ⇒ Object
- #tax_rate ⇒ Object
- #to_s ⇒ Object
- #total ⇒ Object
- #total_label ⇒ Object
-
#transaction_id ⇒ Object
For moneris and moneris_checkout.
- #update_prices! ⇒ Object
Constructor Details
#initialize(atts = nil, &block) ⇒ Order
Effective::Order.new(items: Product.first) Effective::Order.new(items: [Product.first, Product.second], user: User.first) Effective::Order.new(items: Product.first, user: User.first, billing_address: Effective::Address.new, shipping_address: Effective::Address.new)
163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 |
# File 'app/models/effective/order.rb', line 163 def initialize(atts = nil, &block) super(state: EffectiveOrders::PENDING) # Initialize with state: PENDING return self unless atts.present? if atts.kind_of?(Hash) items = Array(atts[:item]) + Array(atts[:items]) self.user = atts[:user] || items.first.try(:user) if (address = atts[:billing_address]).present? self.billing_address = address self.billing_address.full_name ||= user.to_s.presence end if (address = atts[:shipping_address]).present? self.shipping_address = address self.shipping_address.full_name ||= user.to_s.presence end atts.except(:item, :items, :user, :billing_address, :shipping_address).each do |key, value| self.send("#{key}=", value) end add(items) if items.present? else # Attributes are not a Hash self.user = atts.user if atts.respond_to?(:user) add(atts) end self end |
Instance Attribute Details
#confirmed_checkout ⇒ Object
Set on the Checkout Step 1
22 23 24 |
# File 'app/models/effective/order.rb', line 22 def confirmed_checkout @confirmed_checkout end |
#send_mark_as_paid_email_to_buyer ⇒ Object
Set by Admin::Orders#mark_as_paid
26 27 28 |
# File 'app/models/effective/order.rb', line 26 def send_mark_as_paid_email_to_buyer @send_mark_as_paid_email_to_buyer end |
#send_payment_request_to_buyer ⇒ Object
Settings in the /admin action forms
25 26 27 |
# File 'app/models/effective/order.rb', line 25 def send_payment_request_to_buyer @send_payment_request_to_buyer end |
#skip_buyer_validations ⇒ Object
Set by Admin::Orders#create
27 28 29 |
# File 'app/models/effective/order.rb', line 27 def skip_buyer_validations @skip_buyer_validations end |
#terms_and_conditions ⇒ Object
Yes, I agree to the terms and conditions
21 22 23 |
# File 'app/models/effective/order.rb', line 21 def terms_and_conditions @terms_and_conditions end |
Instance Method Details
#abandoned? ⇒ Boolean
416 417 418 |
# File 'app/models/effective/order.rb', line 416 def abandoned? state == EffectiveOrders::ABANDONED end |
#add(*items, quantity: 1) ⇒ Object
Items can be an Effective::Cart, an Effective::order, a single acts_as_purchasable, or multiple acts_as_purchasables add(Product.first) => returns an Effective::OrderItem add(Product.first, current_cart) => returns an array of Effective::OrderItems
224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 |
# File 'app/models/effective/order.rb', line 224 def add(*items, quantity: 1) raise 'unable to alter a purchased order' if purchased? raise 'unable to alter a declined order' if declined? cart_items = items.flatten.flat_map do |item| if item.kind_of?(Effective::Cart) item.cart_items.to_a elsif item.kind_of?(ActsAsPurchasable) Effective::CartItem.new(quantity: quantity, purchasable: item) elsif item.kind_of?(Effective::Order) # Duplicate an existing order self.note_to_buyer ||= item.note_to_buyer self.note_internal ||= item.note_internal self.cc ||= item.cc item.order_items.select { |oi| oi.purchasable.kind_of?(Effective::Product) }.map do |oi| purchasable = oi.purchasable product = Effective::Product.new(name: purchasable.purchasable_name, price: purchasable.price, tax_exempt: purchasable.tax_exempt) # Copy over any extended attributes that may have been created atts = purchasable.dup.attributes.except('name', 'price', 'tax_exempt', 'purchased_order_id').compact atts.each do |k, v| next unless product.respond_to?("#{k}=") && product.respond_to?(k) product.send("#{k}=", v) if product.send(k).blank? end Effective::CartItem.new(quantity: oi.quantity, purchasable: product) end else raise 'add() expects one or more acts_as_purchasable objects, or an Effective::Cart' end end.compact # Make sure to reset stored aggregates self.total = nil self.subtotal = nil self.tax = nil retval = cart_items.map do |item| order_items.build( name: item.name, quantity: item.quantity, price: item.price, tax_exempt: (item.tax_exempt || false), ).tap { |order_item| order_item.purchasable = item.purchasable } end retval.size == 1 ? retval.first : retval end |
#assign_confirmed_if_valid! ⇒ Object
This lets us skip to the confirmed workflow for an admin...
499 500 501 502 503 504 505 506 507 508 |
# File 'app/models/effective/order.rb', line 499 def assign_confirmed_if_valid! return unless pending? self.state = EffectiveOrders::CONFIRMED return true if valid? self.errors.clear self.state = EffectiveOrders::PENDING false end |
#billing_first_name ⇒ Object
372 373 374 |
# File 'app/models/effective/order.rb', line 372 def billing_first_name billing_name.to_s.split(' ').first end |
#billing_last_name ⇒ Object
376 377 378 |
# File 'app/models/effective/order.rb', line 376 def billing_last_name Array(billing_name.to_s.split(' ')[1..-1]).join(' ') end |
#confirm! ⇒ Object
Used by admin checkout only
493 494 495 496 |
# File 'app/models/effective/order.rb', line 493 def confirm! return false if purchased? update!(state: EffectiveOrders::CONFIRMED) end |
#confirmed? ⇒ Boolean
384 385 386 |
# File 'app/models/effective/order.rb', line 384 def confirmed? state == EffectiveOrders::CONFIRMED end |
#custom_order? ⇒ Boolean
A custom order is one that was created by an admin We allow custom orders to have their order items updated
402 403 404 |
# File 'app/models/effective/order.rb', line 402 def custom_order? order_items.all? { |oi| oi.purchasable_type == 'Effective::Product' } end |
#decline!(payment: 'none', provider: 'none', card: 'none', validate: true) ⇒ Object
583 584 585 586 587 588 589 590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 |
# File 'app/models/effective/order.rb', line 583 def decline!(payment: 'none', provider: 'none', card: 'none', validate: true) return false if declined? raise EffectiveOrders::AlreadyPurchasedException.new('order already purchased') if purchased? error = nil assign_attributes( state: EffectiveOrders::DECLINED, purchased_at: nil, payment: payment_to_h(payment), payment_provider: provider, payment_card: (card.presence || 'none'), skip_buyer_validations: true ) Effective::Order.transaction do begin run_purchasable_callbacks(:before_decline) save!(validate: validate) run_purchasable_callbacks(:after_decline) rescue => e self.state = state_was error = e. raise ::ActiveRecord::Rollback end end raise "Failed to decline order: #{error || errors.full_messages.to_sentence}" unless error.nil? true end |
#declined? ⇒ Boolean
412 413 414 |
# File 'app/models/effective/order.rb', line 412 def declined? state == EffectiveOrders::DECLINED end |
#defer!(provider: 'none', email: true) ⇒ Object
572 573 574 575 576 577 578 579 580 581 |
# File 'app/models/effective/order.rb', line 572 def defer!(provider: 'none', email: true) return false if purchased? assign_attributes(state: EffectiveOrders::DEFERRED, payment_provider: provider) save! send_payment_request_to_buyer! if email true end |
#deferred? ⇒ Boolean
388 389 390 |
# File 'app/models/effective/order.rb', line 388 def deferred? state == EffectiveOrders::DEFERRED end |
#done? ⇒ Boolean
396 397 398 |
# File 'app/models/effective/order.rb', line 396 def done? persisted? && (purchased? || declined?) end |
#duplicate ⇒ Object
363 364 365 |
# File 'app/models/effective/order.rb', line 363 def duplicate Effective::Order.new(self) end |
#emails_send_to ⇒ Object
Doesn't control anything. Purely for the flash messaging
618 619 620 |
# File 'app/models/effective/order.rb', line 618 def emails_send_to [email, cc.presence].compact.to_sentence end |
#free? ⇒ Boolean
440 441 442 |
# File 'app/models/effective/order.rb', line 440 def free? total == 0 end |
#in_progress? ⇒ Boolean
392 393 394 |
# File 'app/models/effective/order.rb', line 392 def in_progress? pending? || confirmed? || deferred? end |
#label ⇒ Object
297 298 299 300 301 302 303 304 305 306 307 308 309 |
# File 'app/models/effective/order.rb', line 297 def label if refund? && purchased? 'Refund' elsif purchased? 'Receipt' elsif refund? && (pending? || confirmed?) 'Pending Refund' elsif (pending? || confirmed?) 'Pending Order' else 'Order' end end |
#mark_as_purchased! ⇒ Object
Call this as a way to skip over non consequential orders And mark some purchasables purchased
512 513 514 |
# File 'app/models/effective/order.rb', line 512 def mark_as_purchased! purchase!(skip_buyer_validations: true, email: false, skip_quickbooks: true) end |
#num_items ⇒ Object
448 449 450 |
# File 'app/models/effective/order.rb', line 448 def num_items present_order_items.map { |oi| oi.quantity }.sum end |
#payment_method ⇒ Object
Visa - 1234
326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 |
# File 'app/models/effective/order.rb', line 326 def payment_method return nil unless purchased? provider = payment_provider if ['cheque', 'etransfer', 'phone'].include?(payment_provider) # Normalize payment card card = case payment_card.to_s.downcase.gsub(' ', '').strip when '' then nil when 'v', 'visa' then 'Visa' when 'm', 'mc', 'master', 'mastercard' then 'MasterCard' when 'a', 'ax', 'american', 'americanexpress' then 'American Express' when 'd', 'discover' then 'Discover' else payment_card.to_s end # Try again if card == 'none' && payment['card_type'].present? card = case payment['card_type'].to_s.downcase.gsub(' ', '').strip when '' then nil when 'v', 'visa' then 'Visa' when 'm', 'mc', 'master', 'mastercard' then 'MasterCard' when 'a', 'ax', 'american', 'americanexpress' then 'American Express' when 'd', 'discover' then 'Discover' else payment_card.to_s end end last4 = if payment[:active_card] && payment[:active_card].include?('**** **** ****') payment[:active_card][15,4] end # stripe, moneris, moneris_checkout last4 ||= (payment['f4l4'] || payment['first6last4']).to_s.last(4) [provider.presence, card.presence, last4.presence].compact.join(' - ') end |
#pending! ⇒ Object
This is called from admin/orders#create This is intended for use as an admin action only It skips any address or bad user validations It's basically the same as save! on a new order, except it might send the payment request to buyer
481 482 483 484 485 486 487 488 489 490 |
# File 'app/models/effective/order.rb', line 481 def pending! return false if purchased? self.state = EffectiveOrders::PENDING self.addresses.clear if addresses.any? { |address| address.valid? == false } save! send_payment_request_to_buyer! if send_payment_request_to_buyer? true end |
#pending? ⇒ Boolean
380 381 382 |
# File 'app/models/effective/order.rb', line 380 def pending? state == EffectiveOrders::PENDING end |
#purchasables ⇒ Object
420 421 422 |
# File 'app/models/effective/order.rb', line 420 def purchasables present_order_items.map { |order_item| order_item.purchasable } end |
#purchase!(payment: 'none', provider: 'none', card: 'none', email: true, skip_buyer_validations: false, skip_quickbooks: false) ⇒ Object
Effective::Order.new(items: Product.first, user: User.first).purchase!(email: false)
517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 552 553 |
# File 'app/models/effective/order.rb', line 517 def purchase!(payment: 'none', provider: 'none', card: 'none', email: true, skip_buyer_validations: false, skip_quickbooks: false) return true if purchased? # Assign attributes assign_attributes( state: EffectiveOrders::PURCHASED, skip_buyer_validations: skip_buyer_validations, payment_provider: provider, payment_card: (card.presence || 'none'), purchased_at: (purchased_at.presence || Time.zone.now), payment: payment_to_h(payment) ) begin Effective::Order.transaction do run_purchasable_callbacks(:before_purchase) save! update_purchasables_purchased_order! run_purchasable_callbacks(:after_purchase) end rescue => e Effective::Order.transaction do save!(validate: false) update_purchasables_purchased_order! end raise(e) end send_order_receipts! if email after_commit { sync_quickbooks!(skip: skip_quickbooks) } true end |
#purchased?(provider = nil) ⇒ Boolean
406 407 408 409 410 |
# File 'app/models/effective/order.rb', line 406 def purchased?(provider = nil) return false if (state != EffectiveOrders::PURCHASED) return true if provider.nil? || payment_provider == provider.to_s false end |
#refund? ⇒ Boolean
444 445 446 |
# File 'app/models/effective/order.rb', line 444 def refund? total.to_i < 0 end |
#remove(*items) ⇒ Object
196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 |
# File 'app/models/effective/order.rb', line 196 def remove(*items) raise 'unable to alter a purchased order' if purchased? raise 'unable to alter a declined order' if declined? removed = items.map do |item| order_item = if item.kind_of?(Effective::OrderItem) order_items.find { |oi| oi == item } else order_items.find { |oi| oi.purchasable == item } end raise("Unable to find order item for #{item}") if order_item.blank? order_item end removed.each { |order_item| order_item.mark_for_destruction } # Make sure to reset stored aggregates self.total = nil self.subtotal = nil self.tax = nil removed.length == 1 ? removed.first : removed end |
#send_mark_as_paid_email_to_buyer? ⇒ Boolean
469 470 471 |
# File 'app/models/effective/order.rb', line 469 def send_mark_as_paid_email_to_buyer? EffectiveResources.truthy?(send_mark_as_paid_email_to_buyer) end |
#send_order_receipt_to_admin! ⇒ Object
628 629 630 |
# File 'app/models/effective/order.rb', line 628 def send_order_receipt_to_admin! EffectiveOrders.send_email(:order_receipt_to_admin, self) if purchased? end |
#send_order_receipt_to_admin? ⇒ Boolean
452 453 454 455 |
# File 'app/models/effective/order.rb', line 452 def send_order_receipt_to_admin? return false if free? && !EffectiveOrders.send_order_receipts_when_free EffectiveOrders.send_order_receipt_to_admin end |
#send_order_receipt_to_buyer! ⇒ Object
632 633 634 |
# File 'app/models/effective/order.rb', line 632 def send_order_receipt_to_buyer! EffectiveOrders.send_email(:order_receipt_to_buyer, self) if purchased? end |
#send_order_receipt_to_buyer? ⇒ Boolean
457 458 459 460 |
# File 'app/models/effective/order.rb', line 457 def send_order_receipt_to_buyer? return false if free? && !EffectiveOrders.send_order_receipts_when_free EffectiveOrders.send_order_receipt_to_buyer end |
#send_order_receipts! ⇒ Object
622 623 624 625 626 |
# File 'app/models/effective/order.rb', line 622 def send_order_receipts! send_order_receipt_to_admin! if send_order_receipt_to_admin? send_order_receipt_to_buyer! if send_order_receipt_to_buyer? send_refund_notification! if refund? end |
#send_payment_request_to_buyer! ⇒ Object
636 637 638 |
# File 'app/models/effective/order.rb', line 636 def send_payment_request_to_buyer! EffectiveOrders.send_email(:payment_request_to_buyer, self) unless purchased? end |
#send_payment_request_to_buyer? ⇒ Boolean
462 463 464 465 466 467 |
# File 'app/models/effective/order.rb', line 462 def send_payment_request_to_buyer? return false if free? && !EffectiveOrders.send_order_receipts_when_free return false if refund? EffectiveResources.truthy?(send_payment_request_to_buyer) end |
#send_pending_order_invoice_to_buyer! ⇒ Object
640 641 642 |
# File 'app/models/effective/order.rb', line 640 def send_pending_order_invoice_to_buyer! EffectiveOrders.send_email(:pending_order_invoice_to_buyer, self) unless purchased? end |
#send_refund_notification! ⇒ Object
644 645 646 |
# File 'app/models/effective/order.rb', line 644 def send_refund_notification! EffectiveOrders.send_email(:refund_notification_to_admin, self) if purchased? && refund? end |
#skip_buyer_validations? ⇒ Boolean
473 474 475 |
# File 'app/models/effective/order.rb', line 473 def skip_buyer_validations? EffectiveResources.truthy?(skip_buyer_validations) end |
#skip_quickbooks! ⇒ Object
568 569 570 |
# File 'app/models/effective/order.rb', line 568 def skip_quickbooks! sync_quickbooks!(skip: true) end |
#subtotal ⇒ Object
424 425 426 |
# File 'app/models/effective/order.rb', line 424 def subtotal self[:subtotal] || present_order_items.map { |oi| oi.subtotal }.sum end |
#sync_quickbooks!(skip:) ⇒ Object
We support two different Quickbooks synchronization gems: effective_qb_sync and effective_qb_online
556 557 558 559 560 561 562 563 564 565 566 |
# File 'app/models/effective/order.rb', line 556 def sync_quickbooks!(skip:) if EffectiveOrders.qb_online? skip ? EffectiveQbOnline.skip_order!(self) : EffectiveQbOnline.sync_order!(self) end if EffectiveOrders.qb_sync? skip ? EffectiveQbSync.skip_order!(self) : true # Nothing to do end true end |
#tax ⇒ Object
432 433 434 |
# File 'app/models/effective/order.rb', line 432 def tax self[:tax] || get_tax() end |
#tax_rate ⇒ Object
428 429 430 |
# File 'app/models/effective/order.rb', line 428 def tax_rate self[:tax_rate] || get_tax_rate() end |
#to_s ⇒ Object
293 294 295 |
# File 'app/models/effective/order.rb', line 293 def to_s [label, ' #', to_param].join end |
#total ⇒ Object
436 437 438 |
# File 'app/models/effective/order.rb', line 436 def total (self[:total] || (subtotal + tax.to_i)).to_i end |
#total_label ⇒ Object
311 312 313 314 315 316 317 318 319 320 321 322 323 |
# File 'app/models/effective/order.rb', line 311 def total_label if refund? && purchased? 'Total Paid' elsif purchased? 'Total Paid' elsif refund? && (pending? || confirmed?) 'Total Due' elsif (pending? || confirmed?) 'Total Due' else 'Total' end end |
#transaction_id ⇒ Object
For moneris and moneris_checkout. Just a unique value. Must be 50 characters or fewer or will raise moneris error.
368 369 370 |
# File 'app/models/effective/order.rb', line 368 def transaction_id [to_param, billing_name.to_s.parameterize.first(20).presence, Time.zone.now.to_i, rand(1000..9999)].compact.join('-') end |
#update_prices! ⇒ Object
276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 |
# File 'app/models/effective/order.rb', line 276 def update_prices! raise('already purchased') if purchased? raise('must be pending or confirmed') unless pending? || confirmed? present_order_items.each do |item| purchasable = item.purchasable if purchasable.blank? || purchasable.marked_for_destruction? item.mark_for_destruction else item.assign_purchasable_attributes end end save! end |