Class: Effective::Order
- Inherits:
-
ActiveRecord::Base
- Object
- ActiveRecord::Base
- Effective::Order
- Defined in:
- app/models/effective/order.rb
Instance Attribute Summary collapse
-
#confirmed_checkout ⇒ Object
Set on the Checkout Step 1.
-
#send_mark_as_paid_email_to_buyer ⇒ Object
Set by Admin::Orders#mark_as_paid.
-
#send_payment_request_to_buyer ⇒ Object
Settings in the /admin action forms.
-
#skip_buyer_validations ⇒ Object
Set by Admin::Orders#create.
-
#terms_and_conditions ⇒ Object
Yes, I agree to the terms and conditions.
Instance Method Summary collapse
-
#add(*items, quantity: 1) ⇒ Object
Items can be an Effective::Cart, an Effective::order, a single acts_as_purchasable, or multiple acts_as_purchasables add(Product.first) => returns an Effective::OrderItem add(Product.first, current_cart) => returns an array of Effective::OrderItems.
-
#assign_confirmed_if_valid! ⇒ Object
This lets us skip to the confirmed workflow for an admin...
-
#confirm! ⇒ Object
Used by admin checkout only.
- #confirmed? ⇒ Boolean
- #decline!(payment: 'none', provider: 'none', card: 'none', validate: true) ⇒ Object
- #declined? ⇒ Boolean
- #defer!(provider: 'none', email: true) ⇒ Object
- #deferred? ⇒ Boolean
- #free? ⇒ Boolean
-
#initialize(atts = nil, &block) ⇒ Order
constructor
Effective::Order.new(items: Product.first) Effective::Order.new(items: [Product.first, Product.second], user: User.first) Effective::Order.new(items: Product.first, user: User.first, billing_address: Effective::Address.new, shipping_address: Effective::Address.new).
- #num_items ⇒ Object
-
#pending! ⇒ Object
This is called from admin/orders#create This is intended for use as an admin action only It skips any address or bad user validations It's basically the same as save! on a new order, except it might send the payment request to buyer.
- #pending? ⇒ Boolean
- #purchasables ⇒ Object
-
#purchase!(payment: 'none', provider: 'none', card: 'none', email: true, skip_buyer_validations: false) ⇒ Object
Effective::Order.new(items: Product.first, user: User.first).purchase!(email: false).
- #purchased?(provider = nil) ⇒ Boolean
- #refund? ⇒ Boolean
- #send_mark_as_paid_email_to_buyer? ⇒ Boolean
- #send_order_receipt_to_admin! ⇒ Object
- #send_order_receipt_to_buyer! ⇒ Object
- #send_order_receipts! ⇒ Object
- #send_payment_request_to_buyer! ⇒ Object
- #send_payment_request_to_buyer? ⇒ Boolean
- #send_pending_order_invoice_to_buyer! ⇒ Object
- #send_refund_notification! ⇒ Object
- #skip_buyer_validations? ⇒ Boolean
- #skip_qb_sync! ⇒ Object
- #subtotal ⇒ Object
- #tax ⇒ Object
- #tax_rate ⇒ Object
- #to_s ⇒ Object
- #total ⇒ Object
Constructor Details
#initialize(atts = nil, &block) ⇒ Order
Effective::Order.new(items: Product.first) Effective::Order.new(items: [Product.first, Product.second], user: User.first) Effective::Order.new(items: Product.first, user: User.first, billing_address: Effective::Address.new, shipping_address: Effective::Address.new)
141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 |
# File 'app/models/effective/order.rb', line 141 def initialize(atts = nil, &block) super(state: EffectiveOrders::PENDING) # Initialize with state: PENDING return unless atts.present? if atts.kind_of?(Hash) items = Array(atts.delete(:item)) + Array(atts.delete(:items)) self.user = atts.delete(:user) || (items.first.user if items.first.respond_to?(:user)) if (address = atts.delete(:billing_address)).present? self.billing_address = address self.billing_address.full_name ||= user.to_s.presence end if (address = atts.delete(:shipping_address)).present? self.shipping_address = address self.shipping_address.full_name ||= user.to_s.presence end atts.each { |key, value| self.send("#{key}=", value) } add(items) if items.present? else # Attributes are not a Hash self.user = atts.user if atts.respond_to?(:user) add(atts) end end |
Instance Attribute Details
#confirmed_checkout ⇒ Object
Set on the Checkout Step 1
24 25 26 |
# File 'app/models/effective/order.rb', line 24 def confirmed_checkout @confirmed_checkout end |
#send_mark_as_paid_email_to_buyer ⇒ Object
Set by Admin::Orders#mark_as_paid
28 29 30 |
# File 'app/models/effective/order.rb', line 28 def send_mark_as_paid_email_to_buyer @send_mark_as_paid_email_to_buyer end |
#send_payment_request_to_buyer ⇒ Object
Settings in the /admin action forms
27 28 29 |
# File 'app/models/effective/order.rb', line 27 def send_payment_request_to_buyer @send_payment_request_to_buyer end |
#skip_buyer_validations ⇒ Object
Set by Admin::Orders#create
29 30 31 |
# File 'app/models/effective/order.rb', line 29 def skip_buyer_validations @skip_buyer_validations end |
#terms_and_conditions ⇒ Object
Yes, I agree to the terms and conditions
23 24 25 |
# File 'app/models/effective/order.rb', line 23 def terms_and_conditions @terms_and_conditions end |
Instance Method Details
#add(*items, quantity: 1) ⇒ Object
Items can be an Effective::Cart, an Effective::order, a single acts_as_purchasable, or multiple acts_as_purchasables add(Product.first) => returns an Effective::OrderItem add(Product.first, current_cart) => returns an array of Effective::OrderItems
173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 |
# File 'app/models/effective/order.rb', line 173 def add(*items, quantity: 1) raise 'unable to alter a purchased order' if purchased? raise 'unable to alter a declined order' if declined? cart_items = items.flatten.flat_map do |item| if item.kind_of?(Effective::Cart) item.cart_items.to_a elsif item.kind_of?(ActsAsPurchasable) Effective::CartItem.new(quantity: quantity, purchasable: item) elsif item.kind_of?(Effective::Order) # Duplicate an existing order self.note_to_buyer ||= item.note_to_buyer self.note_internal ||= item.note_internal item.order_items.select { |oi| oi.purchasable.kind_of?(Effective::Product) }.map do |oi| product = Effective::Product.new(name: oi.purchasable.purchasable_name, price: oi.purchasable.price, tax_exempt: oi.purchasable.tax_exempt) Effective::CartItem.new(quantity: oi.quantity, purchasable: product) end else raise 'add() expects one or more acts_as_purchasable objects, or an Effective::Cart' end end.compact # Make sure to reset stored aggregates self.total = nil self.subtotal = nil self.tax = nil retval = cart_items.map do |item| order_items.build( name: item.name, quantity: item.quantity, price: item.price, tax_exempt: (item.tax_exempt || false), ).tap { |order_item| order_item.purchasable = item.purchasable } end retval.size == 1 ? retval.first : retval end |
#assign_confirmed_if_valid! ⇒ Object
This lets us skip to the confirmed workflow for an admin...
310 311 312 313 314 315 316 317 318 319 |
# File 'app/models/effective/order.rb', line 310 def assign_confirmed_if_valid! return unless pending? self.state = EffectiveOrders::CONFIRMED return true if valid? self.errors.clear self.state = EffectiveOrders::PENDING false end |
#confirm! ⇒ Object
Used by admin checkout only
305 306 307 |
# File 'app/models/effective/order.rb', line 305 def confirm! update!(state: EffectiveOrders::CONFIRMED) end |
#confirmed? ⇒ Boolean
229 230 231 |
# File 'app/models/effective/order.rb', line 229 def confirmed? state == EffectiveOrders::CONFIRMED end |
#decline!(payment: 'none', provider: 'none', card: 'none', validate: true) ⇒ Object
374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 |
# File 'app/models/effective/order.rb', line 374 def decline!(payment: 'none', provider: 'none', card: 'none', validate: true) return false if declined? raise EffectiveOrders::AlreadyPurchasedException.new('order already purchased') if purchased? error = nil assign_attributes( state: EffectiveOrders::DECLINED, purchased_at: nil, payment: payment_to_h(payment), payment_provider: provider, payment_card: (card.presence || 'none'), skip_buyer_validations: true ) Effective::Order.transaction do begin save!(validate: validate) rescue => e self.state = state_was error = e. raise ::ActiveRecord::Rollback end end raise "Failed to decline order: #{error || errors..to_sentence}" unless error.nil? run_purchasable_callbacks(:after_decline) true end |
#declined? ⇒ Boolean
243 244 245 |
# File 'app/models/effective/order.rb', line 243 def declined? state == EffectiveOrders::DECLINED end |
#defer!(provider: 'none', email: true) ⇒ Object
359 360 361 362 363 364 365 366 367 368 369 370 371 372 |
# File 'app/models/effective/order.rb', line 359 def defer!(provider: 'none', email: true) return false if purchased? assign_attributes( state: EffectiveOrders::DEFERRED, payment_provider: provider ) save! send_payment_request_to_buyer! if email true end |
#deferred? ⇒ Boolean
233 234 235 |
# File 'app/models/effective/order.rb', line 233 def deferred? state == EffectiveOrders::DEFERRED end |
#free? ⇒ Boolean
267 268 269 |
# File 'app/models/effective/order.rb', line 267 def free? total == 0 end |
#num_items ⇒ Object
275 276 277 |
# File 'app/models/effective/order.rb', line 275 def num_items order_items.map { |oi| oi.quantity }.sum end |
#pending! ⇒ Object
This is called from admin/orders#create This is intended for use as an admin action only It skips any address or bad user validations It's basically the same as save! on a new order, except it might send the payment request to buyer
295 296 297 298 299 300 301 302 |
# File 'app/models/effective/order.rb', line 295 def pending! self.state = EffectiveOrders::PENDING self.addresses.clear if addresses.any? { |address| address.valid? == false } save! send_payment_request_to_buyer! if send_payment_request_to_buyer? true end |
#pending? ⇒ Boolean
225 226 227 |
# File 'app/models/effective/order.rb', line 225 def pending? state == EffectiveOrders::PENDING end |
#purchasables ⇒ Object
247 248 249 |
# File 'app/models/effective/order.rb', line 247 def purchasables order_items.map { |order_item| order_item.purchasable } end |
#purchase!(payment: 'none', provider: 'none', card: 'none', email: true, skip_buyer_validations: false) ⇒ Object
Effective::Order.new(items: Product.first, user: User.first).purchase!(email: false)
322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 |
# File 'app/models/effective/order.rb', line 322 def purchase!(payment: 'none', provider: 'none', card: 'none', email: true, skip_buyer_validations: false) return false if purchased? error = nil assign_attributes( state: EffectiveOrders::PURCHASED, purchased_at: Time.zone.now, payment: payment_to_h(payment), payment_provider: provider, payment_card: (card.presence || 'none'), skip_buyer_validations: skip_buyer_validations ) Effective::Order.transaction do begin run_purchasable_callbacks(:before_purchase) save! update_purchasables_purchased_order! rescue => e self.state = state_was self.purchased_at = nil error = e. raise ::ActiveRecord::Rollback end end raise "Failed to purchase order: #{error || errors..to_sentence}" unless error.nil? send_refund_notification! if email && refund? send_order_receipts! if email run_purchasable_callbacks(:after_purchase) true end |
#purchased?(provider = nil) ⇒ Boolean
237 238 239 240 241 |
# File 'app/models/effective/order.rb', line 237 def purchased?(provider = nil) return false if (state != EffectiveOrders::PURCHASED) return true if provider.nil? || payment_provider == provider.to_s false end |
#refund? ⇒ Boolean
271 272 273 |
# File 'app/models/effective/order.rb', line 271 def refund? total.to_i < 0 end |
#send_mark_as_paid_email_to_buyer? ⇒ Boolean
283 284 285 |
# File 'app/models/effective/order.rb', line 283 def send_mark_as_paid_email_to_buyer? truthy?(send_mark_as_paid_email_to_buyer) end |
#send_order_receipt_to_admin! ⇒ Object
413 414 415 |
# File 'app/models/effective/order.rb', line 413 def send_order_receipt_to_admin! send_email(:order_receipt_to_admin, to_param) if purchased? end |
#send_order_receipt_to_buyer! ⇒ Object
417 418 419 |
# File 'app/models/effective/order.rb', line 417 def send_order_receipt_to_buyer! send_email(:order_receipt_to_buyer, to_param) if purchased? end |
#send_order_receipts! ⇒ Object
408 409 410 411 |
# File 'app/models/effective/order.rb', line 408 def send_order_receipts! send_order_receipt_to_admin! if EffectiveOrders.mailer[:send_order_receipt_to_admin] send_order_receipt_to_buyer! if EffectiveOrders.mailer[:send_order_receipt_to_buyer] end |
#send_payment_request_to_buyer! ⇒ Object
421 422 423 |
# File 'app/models/effective/order.rb', line 421 def send_payment_request_to_buyer! send_email(:payment_request_to_buyer, to_param) unless purchased? end |
#send_payment_request_to_buyer? ⇒ Boolean
279 280 281 |
# File 'app/models/effective/order.rb', line 279 def send_payment_request_to_buyer? truthy?(send_payment_request_to_buyer) && !free? && !refund? end |
#send_pending_order_invoice_to_buyer! ⇒ Object
425 426 427 |
# File 'app/models/effective/order.rb', line 425 def send_pending_order_invoice_to_buyer! send_email(:pending_order_invoice_to_buyer, to_param) unless purchased? end |
#send_refund_notification! ⇒ Object
429 430 431 |
# File 'app/models/effective/order.rb', line 429 def send_refund_notification! send_email(:refund_notification_to_admin, to_param) if purchased? && refund? end |
#skip_buyer_validations? ⇒ Boolean
287 288 289 |
# File 'app/models/effective/order.rb', line 287 def skip_buyer_validations? truthy?(skip_buyer_validations) end |
#skip_qb_sync! ⇒ Object
433 434 435 |
# File 'app/models/effective/order.rb', line 433 def skip_qb_sync! defined?(EffectiveQbSync) ? EffectiveQbSync.skip_order!(self) : true end |
#subtotal ⇒ Object
251 252 253 |
# File 'app/models/effective/order.rb', line 251 def subtotal self[:subtotal] || order_items.map { |oi| oi.subtotal }.sum end |
#tax ⇒ Object
259 260 261 |
# File 'app/models/effective/order.rb', line 259 def tax self[:tax] || get_tax() end |
#tax_rate ⇒ Object
255 256 257 |
# File 'app/models/effective/order.rb', line 255 def tax_rate self[:tax_rate] || get_tax_rate() end |
#to_s ⇒ Object
213 214 215 216 217 218 219 220 221 222 223 |
# File 'app/models/effective/order.rb', line 213 def to_s if refund? "Refund ##{to_param}" elsif purchased? "Receipt ##{to_param}" elsif pending? "Pending Order ##{to_param}" else "Order ##{to_param}" end end |
#total ⇒ Object
263 264 265 |
# File 'app/models/effective/order.rb', line 263 def total (self[:total] || (subtotal + tax.to_i)).to_i end |